!---------------------------------------------------------------------------------------
! FLEXINVERT : add3Dncdf 
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Writes a 3D variable to an existing netcdf file
!---------------------------------------------------------------------------------------

subroutine add3Dncdf(dirout,fileout,varname,varunit,var,nd1,nd2,nd3,&
                     &dim1,dim2,dim3,dname1,dname2,dname3,comment)

  use mod_var

  implicit none

  include 'netcdf.inc'

  character(len=100) :: fileout,dirout
  character(len=100) :: varname,varunit,comment
  character(len=20) :: dname1,dname2,dname3
  integer :: nd1,nd2,nd3,did1,did2,did3
  real :: dim1(nd1),dim2(nd2),dim3(nd3)
  integer :: nid,varid,ierr,ind
  integer :: dimids(3)
  real :: var(nd1,nd2,nd3)

  ! open netcdf file
  write(logid,*) 'Adding to file: '//trim(dirout)//trim(fileout)
  ierr=nf_open(trim(dirout)//trim(fileout),NF_WRITE,nid)
  if(ierr.gt.0) then
    write(logid,*) 'Error opening file: '//trim(dirout)//trim(fileout)
  endif

  ! define dependent variable
  ierr=nf_inq_dimid(nid,dname1,did1)
  ierr=nf_inq_dimid(nid,dname2,did2)
  ierr=nf_inq_dimid(nid,dname3,did3)
  dimids(:)=(/did1,did2,did3/)
  ierr=nf_redef(nid)
  ierr=nf_def_var(nid,varname,NF_FLOAT,3,dimids,varid)
  ind=len_trim(varname)
  ierr=nf_put_att_text(nid,varid,'title',len_trim(varname),varname(1:ind))
  ind=len_trim(varunit)
  ierr=nf_put_att_text(nid,varid,'unit',len_trim(varunit),varunit(1:ind))
  ierr=nf_enddef(nid)

  ! insert variable
  ierr=nf_put_var_real(nid,varid,var)

  ! insert variable attribute
  ierr=nf_redef(nid)
  ind=len_trim(comment)
  ierr=nf_put_att_text(nid,varid,'comment',len_trim(comment),comment(1:ind))
  ierr=nf_enddef(nid)

  ierr=nf_close(nid)

end subroutine add3Dncdf
