!---------------------------------------------------------------------------------------
! FLEXINVERT : backgroundobs
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads observations from ascii files and calculates background
!! at each site using interpolation of lower quartile of observations 
!! in a window of length defined in winback
!---------------------------------------------------------------------------------------

subroutine backgroundobs(allobs,knest_obs,xb)

  use mod_var
  use mod_dates

  implicit none

  integer :: nrec
  character(len=100) :: filename,fileout,dump
  integer, parameter :: maxobs=20000
  integer :: ierr,ierr2,ierr3,i,n,k,j,cutoff,dt,i1,i2,flag,nsteps
  integer :: yyyymmdd,hhmiss,yyyymmddhh
  real(kind=8), dimension(nobsteps,numrec,6) :: allobs
  real, dimension(nbox*nemsteps+nbkgnd) :: xb
  real, dimension(nobsteps*numrec,nemsteps*nbox) :: knest_obs
  integer, dimension(maxobs) :: yr,mn,dd,hh,mi
  integer :: yri,mni,ddi,hhi,mii
  real, dimension(maxobs) :: con,err,con_bg,con_obs
  real :: coni,erri
  real(kind=8) :: jd,jdf
  real(kind=8), dimension(maxobs) :: jdate,date_bg,date_obs
  real, dimension(:), allocatable :: arr,con_n,con_cut,con_sort
  integer, dimension(:), allocatable :: ind,ind_cut

  write(logid,*) 'Calculating background from observations...'

  con(:)=0.
  err(:)=0.
  nrec=0

  open(100,file=trim(dirinvert)//'obslist',action='read',status='old',iostat=ierr)
  if(ierr.gt.0) then
    write(logid,*) 'Error listing files in directory'
  endif

  !---------------------------------------------------------- 
  ! loop over files in directory
  !---------------------------------------------------------- 

  do while(ierr.eq.0)
    read(100,fmt='(A100)',iostat=ierr,end=5) filename
    if(ierr.gt.0) go to 5
    flag=0
    do nrec=1,numrec
      if(filename(1:7).eq.trim(recname(nrec))) flag=1
      if(flag.eq.1) go to 10 
    end do
10  continue

    ! if station is used in inversion open file
    if(flag.gt.0) then
      open(200,file=trim(dirobs)//trim(filename),action='read',status='old',iostat=ierr2)
      if(ierr2.gt.0) write(logid,*) 'Error opening file: '//trim(dirobs)//trim(filename)
      if(ierr2.eq.0) write(logid,*) 'Reading file: '//trim(dirobs)//trim(filename)

      ! modify the lines below for different observation file formats
      do n=1,21
        read(200,fmt='(A100)',iostat=ierr2,end=20) dump
      end do
      i=0
      do while(ierr2.eq.0)
        read(200,fmt='(I4,1X,I2,1X,I2,1X,I2,1X,I2,1X,F10.4,1X,F10.4)',iostat=ierr2,end=20) yri,mni,ddi,hhi,mii,coni,erri
        if(coni.ne.missval) then
          i=i+1
          yr(i)=yri
          mn(i)=mni
          dd(i)=ddi
          hh(i)=hhi
          mi(i)=mii
          con(i)=coni
          err(i)=erri
          yyyymmdd=yr(i)*10000+mn(i)*100+dd(i)
          hhmiss=hh(i)*10000+mi(i)*100
          jdate(i)=juldate(yyyymmdd,hhmiss)
        endif
      end do
20    continue
      close(200)

      if(i.eq.0) go to 30

      n=minloc(jdate,dim=1,mask=jdate.eq.0)-1
      date_bg(:)=0.
      con_bg(:)=0.

      !---------------------------------------------------------- 
      ! average data to obstep
      !----------------------------------------------------------

      ! average observation time interval

      con_obs(:)=0.
      date_obs(:)=0.

      if(rectyp(nrec).eq.'CM') then
        jd=juldatei
        n=0
        do while (jd.le.juldatef)
          jdf=jd+obstep
          i1=minloc(jdate,dim=1,mask=jdate.ge.jd)
          i2=maxloc(jdate,dim=1,mask=jdate.lt.jdf)
          if(i1.lt.i2.and.i1.ne.0.and.i2.ne.0) then
            n=n+1
            con_obs(n)=sum(con(i1:i2))/float(i2-i1+1)
            date_obs(n)=(jd+jdf)/2.
          endif
          jd=jd+obstep
        end do
      else
        con_obs=con
        date_obs=jdate
      end if

      fileout=trim(dirinvert)//filename(1:7)//'_avg.txt'
      open(300,file=trim(fileout),action='write',status='replace',iostat=ierr3)
      do i=1,n
        yyyymmddhh=caldate(date_obs(i))
        write(300,fmt='(I10,1X,F9.3)') yyyymmddhh,con_obs(i)
      end do
      close(300)

      !---------------------------------------------------------- 
      ! method 1: lower quartiles
      !---------------------------------------------------------- 

      if(method.eq.1) then

        ! select lower quarter for each time window
        nsteps=(date_obs(n)-date_obs(1)+1)/winback
        call quartiles(nsteps,n,date_obs(1:n),con_obs(1:n),date_bg(1:n),con_bg(1:n))
        n=minloc(date_bg,dim=1,mask=date_bg.eq.0)-1

      endif ! method 1

      !---------------------------------------------------------- 
      ! method 2: not yet implemented
      !----------------------------------------------------------
      
      if(method.eq.2) then

        write(logid,*) 'Selected method is not yet implemented'
        stop

      endif ! method 2

      !----------------------------------------------------------
      ! subtract prior simulated mixing ratio
      !----------------------------------------------------------

      do i=1,n-1
        if(date_bg(i).ge.juldatei.and.date_bg(i).le.juldatef) then
          j=floor(date_bg(i)-juldatei)+1
          con_bg(i)=con_bg(i)-dot_product(knest_obs(numrec*(j-1)+nrec,:),xb(1:nemsteps*nbox))
        endif
      end do

      fileout=trim(dirinvert)//filename(1:7)//'_bkgnd.txt'
      open(300,file=trim(fileout),action='write',status='replace',iostat=ierr3)
      do i=1,n-1
        yyyymmddhh=caldate(date_bg(i))
        write(300,fmt='(I10,1X,F9.3)') yyyymmddhh,con_bg(i)
      end do
      close(300)

      !----------------------------------------------------------
      ! interpolate background to observation time stamp
      !----------------------------------------------------------

      do i=1,nobsteps
        i1=maxloc(date_bg,dim=1,mask=date_bg.le.allobs(i,nrec,1))
        i2=minloc(date_bg,dim=1,mask=date_bg.ge.allobs(i,nrec,1))
        if(allobs(i,nrec,1).lt.date_bg(1)) then
          i1=1
          i2=1
        endif
        if(allobs(i,nrec,1).gt.date_bg(n)) then
          i1=n
          i2=n
        endif
        if(i1.ne.i2) then 
          allobs(i,nrec,6)=real(con_bg(i1)+(con_bg(i2)-con_bg(i1))*&
                           &(allobs(i,nrec,1)-date_bg(i1))/(date_bg(i2)-date_bg(i1)),kind=8)
        else
          allobs(i,nrec,6)=real(con_bg(i1),kind=8)
        endif 
      end do

    endif ! flag

    ! clean variables for next file
    jdate(:)=0.
    con(:)=0.
    yr(:)=0
    mn(:)=0
    dd(:)=0
    hh(:)=0
    mi(:)=0

30  continue

  end do ! loop over files

  close(100)

5 write(logid,*) 'Error reading from obslist'

end subroutine backgroundobs

