!---------------------------------------------------------------------------------------
! FLEXINVERT : calcmeansurfinf
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads the individual grid_time output files at each timestep and sums the 
!! surface influence over all receptors and time intervals 
!! note: the mean surface influence is used for the calculation of the optimal grid
!---------------------------------------------------------------------------------------

subroutine calcmeansurfinf(surfinf)

  use mod_var
  use mod_dates

  implicit none

  integer :: nr,i
  logical :: lexist
  integer :: yyyymmddhh,yyyymm,eomday,numpoint
  real(kind=8) :: jd
  character(len=100) :: datestr
  character(len=100) :: dirflexrec,filename
  character(len=100) :: varunit,varname,dname1,dname2,comment
  real, dimension(nzmax) :: outheight
  real, dimension(numxgrid,numygrid) :: surfinf
  real, dimension(:,:,:,:), allocatable :: grid,uncert,muncert,mgrid,mgridp

  allocate(grid(numxgrid,numygrid,numzgrid,maxpoint))
  allocate(uncert(numxgrid,numygrid,numzgrid,maxpoint))
  allocate(muncert(numxgrid,numygrid,numzgrid,maxpoint))
  allocate(mgrid(numxgrid,numygrid,numzgrid,maxpoint))
  allocate(mgridp(numxgrid,numygrid,numzgrid,maxpoint))
  surfinf(:,:)=0.

  ! check for existing mean surfinf for this inversion

  write(datesti,'(i8)') datei
  write(datestf,'(i8)') datef
  write(nxst,'(i3.3)') numxgrid
  write(nyst,'(i3.3)') numygrid
  write(nrecst,'(i3.3)') numrec
  filesurfinf='surfinf_'//trim(datesti)//'_'//trim(datestf)//'_'//&
                &trim(nxst)//'x'//trim(nyst)//'x'//trim(nrecst)//'.nc'
  inquire(file=trim(dirauxil)//trim(filesurfinf),exist=lexist)

  if(.not.lexist) then
    ! calculate mean surfinf
    ! loop over stations/receptors
    ! readgrid selects the species specified by 'spec' in control.def
    write(logid,*) 'Average surface influence file not found: '//trim(filesurfinf)
    do nr=1,numrec
      jd=juldatei
      do while (jd.le.juldatef)
        yyyymmddhh=caldate(jd)
        yyyymm=yyyymmddhh/10000
        eomday=calceomday(yyyymm)
        numpoint=eomday*24/relperiod
        write(datestr,'(i6)') yyyymm
        dirflexrec=trim(dirflex)//trim(recname(nr))//'/'//trim(datestr)//'/'
        if(usedirflex) then
          filename=trim(dirflexrec)//'header_grid_time'
        else
          filename=trim(dirflexrec)//'header'
        endif
        call readheader(filename,numpoint,outheight)
        call readgrid(dirflexrec,jd,numpoint,grid,uncert)
        surfinf=surfinf+sum(grid(:,:,1,1:numpoint),dim=3)
        jd=jd+float(abs(loutstep))/3600./24.
      end do
    end do
    ! write surface influence to ncdf file
    varname='surfinf'
    varunit='sm3kg-1'
    dname1='dimx'
    dname2='dimy'
    comment='total surface influence over inversion interval'
    call write2Dncdf(dirauxil,filesurfinf,varname,varunit,surfinf,numxgrid,&
                     &numygrid,dimx,dimy,dname1,dname2,comment)
  else
    ! read existing surfinf
     write(logid,*) 'Reading average surface influence file: '//trim(filesurfinf)
     call readsurfinf(surfinf)
  endif

  deallocate(grid)
  deallocate(uncert)
  deallocate(muncert)
  deallocate(mgrid)
  deallocate(mgridp)

end subroutine calcmeansurfinf


