!---------------------------------------------------------------------------------------
! FLEXINVERT : calcobsvector
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads observations for each receptor and writes these to the observation vector
!! for each observation timestep of the inversion
!---------------------------------------------------------------------------------------

subroutine calcobsvector(knest,knest_fin,kout,knest_obs,knest_finobs,kout_obs,allobs,moderr)

  use mod_var
  use mod_dates

  implicit none

  character(len=100) :: filename,datestr
  integer :: i,j,k,n,nr,ir,indm,it,cnt
  integer :: yyyymmddhh,yyyymm,preyyyymm,yyyy,mm,dd,hh
  integer :: numpoint,eomday
  real(kind=8) :: jd,jdi,jdf,jdn
  real(kind=8), dimension(1000) :: jdobs
  real :: mobs,sdobs,merr,hloc
  real, dimension(nzmax) :: outheight
  integer, dimension(numrel) :: filter
  real, dimension(numrec*(numday*24/relperiod),nemsteps*nbox) :: knest
  real, dimension(numrec*(numday*24/relperiod),numxnest*numynest) :: knest_fin
  real, dimension(numrec*(numday*24/relperiod),ncrs) :: kout
  real, dimension(numday*24/relperiod,numrec) :: moderr
  real, dimension(nobsteps*numrec,nemsteps*nbox) :: knest_obs
  real, dimension(nobsteps*numrec,numxnest*numynest) :: knest_finobs
  real, dimension(nobsteps*numrec,ncrs) :: kout_obs
  real(kind=8), dimension(nobsteps,numrec,6) :: allobs

  write(logid,*) 'Reading observations...'

  jdi=juldatei
  i=0
  ir=0
  indm=0
  ! loop over all observation timesteps in inversion
  do while (jdi.le.juldatef)
    jd=jdi
    yyyymmddhh=caldate(jdi)
    yyyymm=yyyymmddhh/10000
    write(datestr,'(i6)') yyyymm
    if(usedirflex) then
      filename=trim(dirflex)//trim(recname(1))//'/'//trim(datestr)//'/header_grid_time'
    else
      filename=trim(dirflex)//trim(recname(1))//'/'//trim(datestr)//'/header'
    endif
    call readheader(filename,numpoint,outheight)
    ! loop over all observation timesteps in current month
    k=0
    do while (jd.le.(jdi+real(numpoint*relperiod/24)-obstep))
      i=i+1
      if(i.gt.nobsteps) go to 40
      k=k+1
      jdf=jd+obstep
      ! loop over stations/receptors
      do nr=1,numrec
        ir=ir+1
        ! read observation files and do data selection and averaging
        call readobs(recname(nr),rectyp(nr),reclon(nr),recalt(nr),jd,jdf,jdobs,mobs,sdobs,merr,cnt)
        ! date
        allobs(i,nr,1)=(jd+jdf)/2.
        ! observation mean
        allobs(i,nr,2)=real(mobs,kind=8)
        ! observation sd
        allobs(i,nr,3)=real(sdobs,kind=8)
        ! observation error
        allobs(i,nr,4)=real(merr,kind=8)
        ! model error
        allobs(i,nr,5)=real(moderr(i,nr),kind=8)
        ! select/average trajectories in knest, knest_fin and kout to fit observations
        if(rectyp(nr).eq.'FM') then
          ! flasks 
          ! if one data point select closest trajectory
          ! if more than one, average closest trajectories
          if(cnt.gt.0.) then
            do n=1,cnt
              j=floor((jdobs(n)-jdi)*real(24/relperiod))+1
              knest_obs(ir,:)=knest_obs(ir,:)+knest((nr-1)*(numday*24/relperiod)+indm+j,:)
              knest_finobs(ir,:)=knest_finobs(ir,:)+knest_fin((nr-1)*(numday*24/relperiod)+indm+j,:)
              kout_obs(ir,:)=kout_obs(ir,:)+kout((nr-1)*(numday*24/relperiod)+indm+j,:)
            end do
            knest_obs(ir,:)=knest_obs(ir,:)/float(cnt)
            knest_finobs(ir,:)=knest_finobs(ir,:)/float(cnt)
            kout_obs(ir,:)=kout_obs(ir,:)/float(cnt)
          else
            knest_obs(ir,:)=0.
            knest_finobs(ir,:)=0.
            kout_obs(ir,:)=0.
          endif
        endif ! rectyp
        if(rectyp(nr).eq.'CM') then
          ! in-situ
          ! average over observation time resolution
          if(.not.obsfilter) then
            knest_obs(ir,:)=sum(knest((nr-1)*(numday*24/relperiod)+indm+(k-1)*numrel+1:&
                                 &(nr-1)*(numday*24/relperiod)+indm+k*numrel,:),dim=1)/float(numrel)
            knest_finobs(ir,:)=sum(knest_fin((nr-1)*(numday*24/relperiod)+indm+(k-1)*numrel+1:&
                                 &(nr-1)*(numday*24/relperiod)+indm+k*numrel,:),dim=1)/float(numrel)
            kout_obs(ir,:)=sum(kout((nr-1)*(numday*24/relperiod)+indm+(k-1)*numrel+1:&
                                 &(nr-1)*(numday*24/relperiod)+indm+k*numrel,:),dim=1)/float(numrel)
          else
            ! filter to match observation
            ! local time
            it=0
            filter(:)=0
            jdn=jd
            do while (jdn.lt.jdf)
              it=it+1
              ! local time
              hh=floor((jdn-floor(jdn))*24)
              hloc=hh+int(reclon(nr)*24./360.)
              if(hloc.lt.0) hloc=hloc+24
              if(hloc.gt.24) hloc=hloc-24
              if(recalt(nr).gt.1000) then
                ! select nighttime for mountain sites
                if(hloc.le.6) then
                  filter(it)=1
                endif
              else
                ! select afternoon for low altitude sites
                if(hloc.ge.12.and.hloc.lt.18) then
                  filter(it)=1
                endif
              endif ! recalt
              jdn=jdn+float(relperiod)/24.
            end do ! jdn
            if(sum(filter).gt.0) then
              do it=1,numrel
                knest_obs(ir,:)=knest_obs(ir,:)+&
                             &knest((nr-1)*(numday*24/relperiod)+indm+(k-1)*numrel+it,:)*&
                             &float(filter(it))/float(sum(filter))
                knest_finobs(ir,:)=knest_finobs(ir,:)+&
                             &knest_fin((nr-1)*(numday*24/relperiod)+indm+(k-1)*numrel+it,:)*&
                             &float(filter(it))/float(sum(filter))
                kout_obs(ir,:)=kout_obs(ir,:)+&
                             &kout((nr-1)*(numday*24/relperiod)+indm+(k-1)*numrel+it,:)*&
                             &float(filter(it))/float(sum(filter))
              end do
            else
              knest_obs(ir,:)=0.
              knest_finobs(ir,:)=0.
              kout_obs(ir,:)=0.
            endif
          endif ! obsfilter
        endif ! rectyp
      end do ! nr
40    continue
      jd=jd+obstep
    end do ! jd
    indm=indm+numpoint
    eomday=calceomday(yyyymm)
    jdi=jdi+eomday
  end do ! juldatef

  allocate(dimi(ir))

  do n=1,ir
    dimi(n)=n
  enddo

  write(logid,fmt='(1X,A32,1X,I6)') 'Number of observation timesteps:',ir


end subroutine calcobsvector

