!---------------------------------------------------------------------------------------
! FLEXINVERT : calcpriorerror
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Calculates error as a fraction of the maximum value of each pixel and its 
!! 8 surrounding ones, and then sets ocean pixels to the minimum value (minerr)
!---------------------------------------------------------------------------------------

subroutine calcpriorerror(outheight)

  use mod_var

  implicit none

  integer :: jy,ix,i1,i2,j1,j2
  real, dimension(nzmax) :: outheight


  write(logid,*) 'Calculating prior flux errors...'

  allocate(errprior(numxgrid,numygrid))

  ! calculate error as a fraction of the maximum value of each pixel 
  ! and the 8 surrounding ones

  errprior(:,:)=0.
  do jy=1,numygrid
    do ix=1,numxgrid
      i1=max(1,ix-1)
      i2=min(numxgrid,ix+1)
      j1=max(1,jy-1)
      j2=min(numygrid,jy+1)
      errprior(ix,jy)=maxval(emisprior(i1:i2,j1:j2,:))*fracerr
    end do
  end do

  ! set max and min errors and correct for land-sea

  ! convert max and min errors to volume mixing ratio units (kg/m3/s)
  maxerr=maxerr/3600./outheight(1)
  minerr=minerr/3600./outheight(1)

  if(uselsm) then
    ! set errors using land-sea mask 
    do jy=1,numygrid
      do ix=1,numxgrid
        if(lsmflex(ix,jy,1).ge.0.5) then
          ! land
          errprior(ix,jy)=max(errprior(ix,jy),minerr)
          errprior(ix,jy)=min(errprior(ix,jy),maxerr)
        else
          ! sea
          errprior(ix,jy)=minerr
!           errprior(ix,jy)=0.
        endif
      end do
    end do
  else
    ! set errors with no mask
    do jy=1,numygrid
      do ix=1,numxgrid
        errprior(ix,jy)=max(errprior(ix,jy),minerr)
        errprior(ix,jy)=min(errprior(ix,jy),maxerr)
      end do
    end do
  endif ! uselsm

end subroutine calcpriorerror

