!---------------------------------------------------------------------------------------
! FLEXINVERT : calcstatevector
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Interpolates the prior fluxes onto the inversion flux time stamp and maps
!! these fluxes to the inversion grid 
!---------------------------------------------------------------------------------------

subroutine calcstatevector(xb,timeout)

  use mod_var
  use mod_dates
 
  implicit none

  integer :: n,nt,i1,i2,ix,jy
  integer :: yyyymmddhh,yyyy,mm,dd,hh
  real(kind=8) :: jd,jtime
  real, dimension(numxgrid*numygrid) :: emisvec
  real, dimension(nemsteps) :: timeout
  real, dimension(nbox*nemsteps+nbkgnd) :: xb

  ! loop over timesteps in inversion and interpolate
  ! prior fluxes to inversion flux time stamp

  emisprior(:,:,:)=0.
  timeout(:)=0.

  do n=1,nemsteps
    jd=juldatei+(float(n)-1.)*tstep
    yyyymmddhh=caldate(jd)
    yyyy=yyyymmddhh/1000000
    mm=(yyyymmddhh-yyyy*1000000)/10000
    dd=(yyyymmddhh-yyyy*1000000-mm*10000)/100
    hh=yyyymmddhh-yyyy*1000000-mm*10000-dd*100
    jtime=float(yyyy)+float(mm-1)/12.+float(dd-1)/365.+float(hh)/365./24.
    timeout(n)=jtime
    nt=minloc(abs(timemi(:)-jtime),dim=1,mask=abs(timemi(:)-jtime).eq.minval(abs(timemi(:)-jtime)))
    if(timemi(nt).ge.jtime) then
      i1=nt-1
      i2=nt
    else
      i1=nt
      i2=nt+1
    end if
    i1=max(1,i1)
    i2=min(ntemi,i2)
    do ix=1,numxgrid
      do jy=1,numygrid
        if(i1.ne.i2) then
          emisprior(ix,jy,n)=emisflex(ix,jy,i1)+(emisflex(ix,jy,i2)-emisflex(ix,jy,i1))&
                           &*(jtime-timemi(i1))/(timemi(i2)-timemi(i1))
        else
          emisprior(ix,jy,n)=emisflex(ix,jy,nt)
        end if
        emisvec((jy-1)*numxgrid+ix)=emisprior(ix,jy,n)*area_grid(ix,jy)
      end do
    end do
    ! project fine grid fluxes onto variable grid
    xb((n-1)*nbox+1:n*nbox)=matmul(real(gridoper),emisvec)/area_box
  end do  

  ! if optimizing background then add prior scalar values

  if(optimizbg) then
    do n=1,nbkgnd
      xb(nbox*nemsteps+n)=1.
    end do
  endif

  ! dimension vector of xb
 
  allocate(dimb(nbox*nemsteps))
  allocate(dimbx(nbox*nemsteps+nbkgnd))

  do n=1,nbox*nemsteps
    dimb(n)=n
  end do

  do n=1,nbox*nemsteps+nbkgnd
    dimbx(n)=n
  end do

end subroutine calcstatevector


