!---------------------------------------------------------------------------------------
! FLEXINVERT : correl
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Calculates spatial correlation between grid cells 
!! allows for separate correlation lengths for land and sea
!---------------------------------------------------------------------------------------

subroutine correl(cov)

  use mod_var

  implicit none

  integer, parameter :: dj=60
  integer :: i1,i2,j1,j2,ind1,ind2
  real :: pih,cor,sigma,dd,dx
  real :: lat1,lat2,lon1,lon2
  real, dimension(numxnest*numynest,numxnest*numynest) :: cov

  pih=pi/180.
  cov(:,:)=0.

  do j1=1,numynest
    lat1=lly+(j1-1)*dyout
    do i1=1,numxnest
      ind1=(j1-1)*numxnest+i1
      lon1=llx+(i1-1)*dxout
      if(lsmflex(i1,j1,1).ge.0.5) then
        ! convert to meters
        sigma=sigmaland*1000.
      else
        sigma=sigmasea*1000.
      endif
      do j2=j1-dj/2,j1+dj/2,1
        if(j2.ge.1.and.j2.le.numynest) then
          lat2=lly+(j2-1)*dyout
          do i2=1,numxnest
            lon2=llx+(i2-1)*dxout
            ind2=(j2-1)*numxnest+i2
            if(i1.eq.i2.and.j1.eq.j2) then
              dx=0.
            else
              dd=(1-sin(lat2*pih)*sin(lat1*pih)-&
                  &cos(lat2*pih)*cos(lat1*pih)*cos((lon2-lon1)*pih))/2.
              dd=max(dd,0.)
              dx=rearth*2.*asin(sqrt(dd))
            endif
            cor=0.
            if(nint(lsmflex(i1,j1,1)).eq.nint(lsmflex(i2,j2,1))) then
              cor=exp(-dx/sigma)
              if(cor.lt.0.0001) cor=0.
            endif
            ! and to be sure
            if(i1.eq.i2.and.j1.eq.j2) cor=1.
            cov(ind1,ind2)=cor
            cov(ind2,ind1)=cor
          end do
        endif
      end do
    end do
  end do

end subroutine correl
