!---------------------------------------------------------------------------------------
! FLEXINVERT : correl_coarse
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Calculates spatial correlation between grid boxes (coarse resolution) 
!---------------------------------------------------------------------------------------

subroutine correl_coarse(corspace)

  use mod_var

  implicit none

  integer, parameter :: dj=60
  integer :: ix,jy
  real :: indx,indy,num
  integer :: ind1,ind2
  real :: pih,cor,sigma,dd,dx
  real :: lat1,lat2,lon1,lon2
  real, dimension(nbox,nbox) :: corspace


  pih=pi/180.
  sigma=sigmaland*1000.

  do ind1=1,nbox
    indx=0.
    indy=0.
    num=0.
    do jy=1,numynest
      do ix=1,numxnest
        if(nbox_xy(ix,jy).eq.ind1) then
          indx=indx+real(ix)
          indy=indy+real(jy)
          num=num+1.
        end if
      end do
    end do
    indx=nint(indx/num)
    indy=nint(indy/num)
    lon1=llx+(indx-1.)*dxout
    lat1=lly+(indy-1.)*dyout
    do ind2=ind1,nbox
      indx=0.
      indy=0.
      num=0.
      do jy=1,numynest
        do ix=1,numxnest
          if(nbox_xy(ix,jy).eq.ind2) then
            indx=indx+real(ix)
            indy=indy+real(jy)
            num=num+1.
          end if
        end do
      end do
      indx=nint(indx/num)
      indy=nint(indy/num)
      lon2=llx+(indx-1.)*dxout
      lat2=lly+(indy-1.)*dyout
      if(ind1.eq.ind2) then
        dx=0.
      else
        dd=(1.-sin(lat2*pih)*sin(lat1*pih)-&
            &cos(lat2*pih)*cos(lat1*pih)*cos((lon2-lon1)*pih))/2.
        dd=max(dd,0.)
        dx=rearth*2.*asin(sqrt(dd))
      endif
      cor=exp(-dx/sigma)
      if(cor.lt.0.0001) cor=0.
      ! and to be sure
      if(ind1.eq.ind2) cor=1.
      corspace(ind1,ind2)=cor
      corspace(ind2,ind1)=cor
    end do ! ind2
  end do ! ind1

end subroutine correl_coarse
