!---------------------------------------------------------------------------------------
! FLEXINVERT : gridcoarse 
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Calculates the interpolation operator for the interpolation from the FLEXPART
!! grid to the coarse grid used in the optimization of the background 
!---------------------------------------------------------------------------------------

subroutine gridcoarse()

  use mod_var

  implicit none

  integer, parameter :: xres=30,yres=15   ! coarse grid size in degrees
  integer :: numx,numy
  integer :: n,i1,j1,i2,j2,ij

  numxcrs=nint(numxgrid*dxout/xres)
  numycrs=nint(numygrid*dyout/yres)
  numy=yres/dyout
  numx=xres/dxout
  ncrs=numxcrs*numycrs
 
  allocate(gridcrs(ncrs,numxgrid*numygrid))
  allocate(area_crs(ncrs))
  gridcrs(:,:)=0
  area_crs(:)=0.

  do j1=1,numygrid,numy
    do i1=1,numxgrid,numx
      ij=floor(real(j1-1)/numy)*numxcrs+floor(real(i1-1)/numx)+1
      do j2=1,numy
        do i2=1,numx
          if((i1+i2-1).gt.numxgrid) go to 10
          gridcrs(ij,(j1+j2-2)*numxgrid+i1+i2-1)=1
          area_crs(ij)=area_crs(ij)+area_grid(i1+i2-1,j1+j2-1)
        end do
10 continue
      end do
    end do
  end do 

  allocate(dimcrs(ncrs))
  do n=1,ncrs
    dimcrs(n)=n
  end do


end subroutine gridcoarse
