!---------------------------------------------------------------------------------------
! FLEXINVERT : initial
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Calls a subroutine to read global fields of initial conditions from files and
!! converts the mass mixing ratios from these to the FLEXPART grid
!---------------------------------------------------------------------------------------

subroutine initial(outheight,ntmax,numpoint,jd,immr,immrtime)

  use mod_var
  use mod_dates

  implicit none

  include 'netcdf.inc'

  character(len=100) :: fileout,dname1,dname2,dname3,varunit,varname,comment
  character(len=4) :: yyyystr,mmstr
  integer :: yyyymmddhh,yyyymmdd,yyyy,mm,it,iz,jy,ix
  real(kind=8) :: jd
  integer :: numpoint,itime,ntmax
  real, dimension(numxgrid,numygrid,numzgrid,numpoint) :: grid
  real, dimension(nzmax) :: outheight
  real, dimension(numxgrid) :: outlon
  real, dimension(numygrid) :: outlat
  real, dimension(numzgrid) :: presout
  real, dimension(:,:,:), allocatable :: immrin,immrout
  real, dimension(:,:,:,:), allocatable :: immrflex
  real, dimension(ntmax) :: immrtime
  real, dimension(numxgrid,numygrid,numzgrid,ntmax) :: immr
  real :: dz,tz,pz

  ! initialize variables
  immrtime(:)=0.
  immr(:,:,:,:)=0.

  !--------------------------------------------------------------
  ! read initial concentration fields
  !--------------------------------------------------------------

  ! select model output to read
!  call readlmdz(jd)
!  call readpyvar(jd)
  call readtm5(jd)

  !--------------------------------------------------------------
  ! convert initial mixing ratios to flexpart grid
  !--------------------------------------------------------------

  immrtime(1:ntini)=timini(1:ntini)

  allocate(immrin(nxini,nyini,1))
  allocate(immrout(numxgrid,numygrid,1))
  allocate(immrflex(numxgrid,numygrid,(nzini-1),ntini))

  ! convert from volume mixing ratio to mass mixing ratio
  immrraw=immrraw*molarmass/mmair

  do it=1,ntini
    do iz=1,nzini-1
      immrin(:,:,1)=immrraw(:,:,iz,it)
      tz=temp(iz)
      if(iz.eq.1) then
        dz=(gasc*tz/(gc*mmol))*log(ps/(0.5*(pres(iz)+pres(iz+1))))
      else
        dz=(gasc*tz/(gc*mmol))*log((pres(iz-1)+pres(iz))/(pres(iz)+pres(iz+1)))
      endif
      call convertgrid(.false.,1,outheight,nxini,nyini,lonini,latini,1,immrin,immrout)
      immrflex(:,:,iz,it)=immrout(:,:,1)     
    end do 
    ! interpolate mass vertically onto flexpart grid
    call interpz(immrflex(:,:,:,it),outheight,immr(:,:,:,it))
  end do 

  ! convert from mass mixing ratio to volume mixing ratio 
  immr=immr*mmair/molarmass 

  ! calculate pressure out 
  do iz=1,numzgrid
    if(iz.eq.1) dz=outheight(iz)
    if(iz.ge.2) dz=outheight(iz)-outheight(iz-1)
    tz=ts-lrate*outheight(iz)
    pz=ps*exp(-outheight(iz)*mmol*gc/(gasc*tz))
    presout(iz)=pz
  end do

  ! for testing save mixing ratios on flexpart grid
  fileout='immr.nc'
  varname='tracer'
  varunit='vmr'
  dname1='longitude'
  dname2='latitude'
  dname3='pressure'
  comment='initial mixing ratios on flexpart grid'
  do ix=1,numxgrid
    outlon(ix)=outlon0+(ix-1)*dxout
  end do
  do jy=1,numygrid
    outlat(jy)=outlat0+(jy-1)*dyout
  end do
  call write3Dncdf(dirinvert,fileout,varname,varunit,immr(:,:,:,1),&
                   &numxgrid,numygrid,numzgrid,outlon,outlat,presout,&
                   &dname1,dname2,dname3,comment)

  deallocate(lonini)
  deallocate(latini)
  deallocate(pres)
  deallocate(timini)
  deallocate(temp)
  deallocate(immrraw)
  deallocate(immrin)
  deallocate(immrout)
  deallocate(immrflex)

end subroutine initial
