!---------------------------------------------------------------------------------------
! FLEXINVERT : interpz
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Interpolate a 3D grid vertically 
!---------------------------------------------------------------------------------------

subroutine interpz(gridin,outheight,gridout)

  use mod_var

  implicit none

  integer :: ix,jy,kz,z,z1,z2
  real :: pz,tz,ht
  real, dimension(nzmax) :: outheight
  real, dimension(numxgrid,numygrid,nzini) :: gridin
  real, dimension(numxgrid,numygrid,numzgrid) :: gridout
 
  do kz=1,numzgrid
    ht=outheight(kz)
    ! calculate temperature using lapse rate
    tz=ts-lrate*ht
    pz=ps*exp(-ht*mmol*gc/(gasc*tz))
    z=minloc(abs(pres(:)-pz),dim=1,mask=abs(pres(:)-pz).eq.minval(abs(pres(:)-pz)))
    if(pres(z).gt.pz) then
      z1=z
      z2=min(z+1,nzini)
    else
      z1=max(z-1,1)
      z2=z  
    endif
    do jy=1,numygrid
      do ix=1,numxgrid
        if(z1.eq.z2) then
          gridout(ix,jy,kz)=gridin(ix,jy,z)
        else  
          gridout(ix,jy,kz)=gridin(ix,jy,z1)+(gridin(ix,jy,z2)-gridin(ix,jy,z1))&
                            &*(pz-pres(z1))/(pres(z2)-pres(z1))
        endif
      end do
    end do
  end do

end subroutine interpz


