!---------------------------------------------------------------------------------------
! FLEXINVERT : mod_var 
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------

module mod_var

  implicit none

  !---------------------------------------------------------------
  ! Definition variables
  !---------------------------------------------------------------

  ! inversion constants
  integer :: datei,datef,tstep,relperiod,maxrec
  integer :: nlatbin,bgmethod,method,winback,seed
  real :: lly,ury,llx,urx
  real :: obstep
  real, dimension(:), allocatable :: latbin
  real :: measerr,bkgerr,fracerr,minerr,maxerr,globerr,sigmaland,sigmasea,sigmatime
  logical :: useemission,uselanduse,uselsm,obsfilter,perturb,optimizbg,scaleflx,checkprior

  ! FLEXPART constants
  integer :: trajlen,nzmax,maxpoint,maxspec,spec
  real :: molarmass,coef
 
  ! path constants
  character(len=100) :: dirinvert,dirflex,dirflexinit,dirobs,dirauxil
  logical :: usedirflex

  ! file constants
  character(len=100) :: logfile,stnfile,fileemis,fileimmr,filelanduse,filelsm
  character(len=100) :: emisname,lsmname,bginame
  logical :: midpoints
  integer :: nclass
  real :: missval

  !---------------------------------------------------------------
  ! File and path variables
  !---------------------------------------------------------------

  character(len=100) :: fileinit,filesurfinf,filecorrel
  integer :: logid

  !---------------------------------------------------------------
  ! Physical and other constants
  !---------------------------------------------------------------

  real, parameter :: numscale=1.e12  ! numeric scaling factor

  real, parameter :: pi=3.14159265  ! pi
  real, parameter :: rearth=6.371e6 ! radius of earth (m)
  real, parameter :: mmair=28.97    ! standard molecular mass of air
  real, parameter :: gasc=8.314     ! idea gas constant (J/K/mol)
  real, parameter :: ps=101325      ! surface pressure (Pa)
  real, parameter :: gc=9.81        ! gravitation (m/s)
  real, parameter :: ts=288.15      ! surface temperature (K)
  real, parameter :: lrate=0.0065   ! lapse rate (K/m)
  real, parameter :: mmol=0.02894   ! (kg/mol)

  !---------------------------------------------------------------
  ! FLEXPART variables
  !---------------------------------------------------------------
 
  integer :: loutstep,loutaver,loutsample
  integer :: numxgrid,numygrid,numzgrid 
  real :: outlon0,outlat0,dxout,dyout
  integer :: nspec

  !---------------------------------------------------------------
  ! Time variables
  !---------------------------------------------------------------

  character(len=100) :: datesti,datestf
  real(kind=8) :: juldatei,juldatef

  !---------------------------------------------------------------
  ! Dimension and geographical variables
  !---------------------------------------------------------------

  integer :: nxlsm,nylsm,nxemi,nyemi,ntemi
  character(len=100) :: nxst,nyst,sigst
  integer :: nxini,nyini,nzini,ntini

  real, dimension(:,:), allocatable :: area_grid
  real, dimension(:,:,:), allocatable :: xlanduse
  real, dimension(:), allocatable :: gridlon,gridlat,lonlsm,latlsm
  real, dimension(:), allocatable :: nestlon,nestlat
  real, dimension(:), allocatable :: lonemi,latemi,timemi
  real, dimension(:), allocatable :: lonini,latini,timini,pres,temp
  real, dimension(:), allocatable :: area_box,area_crs,dimx,dimy,dimxy,dimr,dimi,dimo,dimbox,dimb,dimcrs
  real, dimension(:), allocatable :: dimxnest,dimynest,dimxynest,dimbx
  real, dimension(:,:,:), allocatable :: lsm,lsmflex
  real, dimension(:,:,:,:), allocatable :: immrraw

  !---------------------------------------------------------------
  ! Inversion variables
  !---------------------------------------------------------------
  
  integer :: nbox,ngrids,ncrs,nbkgnd
  integer :: numrec,nobsteps,nemsteps,numrel,numyr,nummon,numday
  integer :: ixr,ixl,iyu,iyl
  integer :: numxnest,numynest
  integer :: numxcrs,numycrs

  integer, dimension(:,:), allocatable :: nbox_xy,gridoper,gridnest,gridcrs
  real, dimension(:,:,:), allocatable :: emis,emisflex,emisprior,emispost,errpost,errpri
  real, dimension(:,:), allocatable :: errprior
  real :: erscalar

  !---------------------------------------------------------------
  ! Observation data variables
  !---------------------------------------------------------------

  character(len=100) :: nrecst
  character(len=20), dimension(:), allocatable :: recname,rectyp
  real, dimension(:), allocatable :: reclon,reclat
  integer, dimension(:), allocatable :: recalt


end module mod_var
