!---------------------------------------------------------------------------------------
! FLEXINVERT: modelerror
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Calculates the model uncertainty for a given observation site using
!! the uncertainties in the gridded FLEXPART output and the prior
!! emissions fields
!---------------------------------------------------------------------------------------

subroutine modelerror(jd,uncert,err)

  use mod_var
  use mod_dates

  implicit none

  integer :: ix,jy,nt
  integer :: yyyymmddhh,yyyy,mm,dd,hh
  real(kind=8) :: jd
  real :: jtime,err
  real, dimension(numxgrid,numygrid) :: uncert

  err=0.

  yyyymmddhh=caldate(jd)
  yyyy=yyyymmddhh/1000000
  mm=(yyyymmddhh-yyyy*1000000)/10000
  dd=(yyyymmddhh-yyyy*1000000-mm*10000)/100
  hh=yyyymmddhh-yyyy*1000000-mm*10000-dd*100
  jtime=yyyy+(mm-1.)/12.+(dd-1.)/365.+hh/365./24.
  nt=minloc(abs(timemi(:)-jtime),dim=1,mask=abs(timemi(:)-jtime).eq.minval(abs(timemi(:)-jtime)))

  do jy=1,numygrid
    do ix=1,numxgrid
      if(.not.(isnan(uncert(ix,jy)).or.isnan(emisflex(ix,jy,nt)))) then
        err=err+uncert(ix,jy)*emisflex(ix,jy,nt)
      endif
    end do
  end do
  
  
end subroutine modelerror

