!---------------------------------------------------------------------------------------
! FLEXINVERT : quartiles
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Sorts concentrations and selects lower quarter for each time window
!---------------------------------------------------------------------------------------

subroutine quartiles(nsteps,nbcon,jdate,con,date_lq,con_lq)

  use mod_var

  implicit none

  integer :: nsteps,nbcon
  integer :: cutoff,n,i,i1,i2,ii,k,j
  real, dimension(nbcon) :: con,con_lq
  real(kind=8), dimension(nbcon) :: jdate,date_lq
  real, dimension(:), allocatable :: arr,arro,con_n,con_cut,con_sort
  real(kind=8), dimension(:), allocatable :: jdate_n,date_cut,date_sort
  integer, dimension(:), allocatable :: ind,ind_cut

  ii=1
  do n=1,nsteps
    i1=minloc(jdate,dim=1,mask=jdate.ge.(jdate(1)+(n-1)*winback))
    i2=maxloc(jdate,dim=1,mask=jdate.lt.(jdate(1)+n*winback))
    ! if too few points make no selection
    if((i2-i1+1).lt.4) go to 15
    allocate(jdate_n(i2-i1+1))
    allocate(con_n(i2-i1+1))
    allocate(arr(i2-i1+1))
    allocate(arro(i2-i1+1))
    allocate(ind(i2-i1+1))
    jdate_n=jdate(i1:i2)
    con_n=con(i1:i2)
    arr=con(i1:i2)
    ! store unsorted
    arro=arr
    call sort(i2-i1+1,arr)
    do k=1,(i2-i1+1)
      ind(k)=minloc(arr,dim=1,mask=arr.eq.arro(k))
    end do
    ! cutoff point of lower quartile
    cutoff=int((i2-i1+1)/4)
    allocate(con_cut(cutoff))
    allocate(ind_cut(cutoff))
    allocate(date_cut(cutoff))
    allocate(con_sort(cutoff))
    allocate(date_sort(cutoff))
    con_cut=arr(1:cutoff)
    ind_cut=ind(1:cutoff)
    do k=1,cutoff
      date_cut(k)=jdate_n(ind_cut(k))
    end do
    ! put back in chronological order
    j=0
    do k=1,maxval(ind_cut)
      i=minloc(ind_cut,dim=1,mask=ind_cut.eq.k)
      if(i.ne.0) then
        j=j+1
        con_sort(j)=con_cut(i)
        date_sort(j)=date_cut(i)
      endif
    end do
    con_lq(ii:(ii+j-1))=con_sort(1:j)
    date_lq(ii:(ii+j-1))=date_sort(1:j)
    ii=ii+j
    deallocate(con_cut)
    deallocate(ind_cut)
    deallocate(date_cut)
    deallocate(con_sort)
    deallocate(date_sort)
    deallocate(jdate_n)
    deallocate(con_n)
    deallocate(arr)
    deallocate(arro)
    deallocate(ind)
15  continue
    if((i2-i1+1).lt.4) then
      j=i2-i1+1
      date_lq(ii:(ii+j-1))=jdate(i1:i2)
      con_lq(ii:(ii+j-1))=con(i1:i2)
      ii=ii+j
    endif
  end do ! nsteps

end subroutine quartiles

