!---------------------------------------------------------------------------------------
! FLEXINVERT : readgrid
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Based on readgrid.f routine of A. Stohl
!!
!! Reads a gridded FLEXPART output file (unformatted binary)
!! at a given time step (jd) and returns an array (grid) of
!! dimensions numxgrid,numygrid,numpoint (for a single species)
!! note: for a backwards run number of releases = numpoint
!---------------------------------------------------------------------------------------

subroutine readgrid(dirflexrec,jd,numpoint,grid,uncert)

  use mod_var
  use mod_dates

  implicit none

  real, parameter :: scaleconc=1.e12
  real, parameter :: smallnum=1.e-38
  character(len=100) :: dirflexrec,filename
  character(len=14) :: datestr
  integer :: yyyymmddhh
  real(kind=8) :: jd
  integer :: numpoint,itime
  integer :: ierr,ks,kp,sp_count_i,sp_count_r,ii,ir,fact,n,kz,jy,ix
  real, dimension(numxgrid,numygrid,numzgrid,maxpoint) :: grid,uncert
  integer, dimension(numxgrid*numygrid*numzgrid) :: sparse_dump_i
  real, dimension(numxgrid*numygrid*numzgrid) :: sparse_dump_r,sparse_dump_u

  yyyymmddhh=caldate(jd)
  write(datestr,'(i10)') yyyymmddhh
  filename=trim(dirflexrec)//'grid_time_'//trim(datestr)//'0000_001'
  filename=trim(filename)

  grid(:,:,:,:)=0.
  open(100,file=filename,form='unformatted',action='read',status='old',iostat=ierr)
  if(ierr.gt.0) write(logid,*) 'Error opening file: '//trim(filename)
  read(100) itime
  do ks=1,nspec
    do kp=1,numpoint
      ! read wet deposition
      read(100) sp_count_i
      read(100) (sparse_dump_i(ix),ix=1,sp_count_i)
      read(100) sp_count_r
      read(100) (sparse_dump_r(ix),ix=1,sp_count_r)
      read(100) (sparse_dump_u(ix),ix=1,sp_count_r)
      ! read dry deposition
      read(100) sp_count_i
      read(100) (sparse_dump_i(ix),ix=1,sp_count_i)
      read(100) sp_count_r
      read(100) (sparse_dump_r(ix),ix=1,sp_count_r)
      read(100) (sparse_dump_u(ix),ix=1,sp_count_r)
      ! read concentrations
      fact=1
      read(100) sp_count_i
      read(100) (sparse_dump_i(ix),ix=1,sp_count_i)
      read(100) sp_count_r
      read(100) (sparse_dump_r(ix),ix=1,sp_count_r)
      read(100) (sparse_dump_u(ix),ix=1,sp_count_r)
      ii=0
      do ir=1,sp_count_r
        if ((sparse_dump_r(ir)*fact).gt.smallnum) then
          ii=ii+1
          n=sparse_dump_i(ii)
          fact=fact*(-1.)
        else
          n=n+1
        endif
        kz=n/(numxgrid*numygrid)
        jy=(n-kz*numxgrid*numygrid)/numxgrid
        ix=n-numxgrid*numygrid*kz-numxgrid*jy
        if(ks.eq.spec) then  
          grid(ix+1,jy+1,kz,kp)=abs(sparse_dump_r(ir))*scaleconc
          uncert(ix+1,jy+1,kz,kp)=abs(sparse_dump_u(ir))*scaleconc
        endif
      end do ! sp_count_r
    end do ! numpoint
  end do ! nspec
  close(100)

end subroutine readgrid
