!---------------------------------------------------------------------------------------
! FLEXINVERT : readheader
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Based on readheader.f routine of A. Stohl
!!
!! Reads header file in FLEXPART output directory
!---------------------------------------------------------------------------------------

subroutine readheader(filename,numpoint,outheight)
 
  use mod_var

  implicit none

  character(len=100) :: filename
  integer :: ibdate,ibtime,nageclass,imethod
  integer :: jjjjmmdd,ihmmss,numpoint
  real :: xp2,yp2
  real, dimension(nzmax) :: outheight
  character(len=10), dimension(maxspec) :: species
  integer, dimension(:), allocatable :: ireleasestart,ireleaseend
  integer, dimension(:), allocatable :: npart,kind
  character(len=45), dimension(:), allocatable :: compoint
  real, dimension(:), allocatable :: xpoint,ypoint,zpoint1,zpoint2
  real, dimension(:,:), allocatable :: xmass
  integer, dimension(10) :: lage
  integer :: n,i,j,ierr

  write(logid,*) 'Reading file: '//trim(filename)

  open(100,file=filename,form='unformatted',action='read',status='old',iostat=ierr)
  if(ierr.gt.0) write(logid,*) 'Error opening file: '//trim(filename)
  read(100) ibdate,ibtime
  read(100) loutstep,loutaver,loutsample
  read(100) outlon0,outlat0,numxgrid,numygrid,dxout,dyout
  read(100) numzgrid, (outheight(i),i=1,numzgrid) 
  read(100) jjjjmmdd,ihmmss
  read(100) nspec,numzgrid
  nspec=nspec/3
  do n=1,nspec
    read(100) numzgrid,species(n)
    read(100) numzgrid,species(n)
    read(100) numzgrid,species(n)
  end do
  read(100) numpoint

  allocate(ireleasestart(numpoint))
  allocate(ireleaseend(numpoint))
  allocate(xpoint(numpoint))
  allocate(ypoint(numpoint))
  allocate(zpoint1(numpoint))
  allocate(zpoint2(numpoint))
  allocate(npart(numpoint))
  allocate(kind(numpoint))
  allocate(xmass(numpoint,nspec))
  allocate(compoint(numpoint))

  do i=1,numpoint
    read(100) ireleasestart(i),ireleaseend(i)
    read(100) xpoint(i),ypoint(i),xp2,yp2,zpoint1(i),zpoint2(i)
    read(100) npart(i),kind(i)
    read(100) compoint(i)
     do j=1,nspec
       read(100)
       read(100)
       read(100) xmass(i,j)
     end do
  end do
  read(100) imethod
  read(100) nageclass,(lage(i),i=1,nageclass)
  close(100)

  deallocate(ireleasestart)
  deallocate(ireleaseend)
  deallocate(xpoint)
  deallocate(ypoint)
  deallocate(zpoint1)
  deallocate(zpoint2)
  deallocate(npart)
  deallocate(kind)
  deallocate(xmass)
  deallocate(compoint)

end subroutine readheader

