!---------------------------------------------------------------------------------------
! FLEXINVERT : readinit
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads a gridded FLEXPART file (unformatted binary) of the sensitivity to the
!! initial conditions (grid_initial_001)
!! returns an array (grid) of dimensions numxgrid,numygrid,numpoint (for a single species)
!! note: for a backwards run number of releases = numpoint
!---------------------------------------------------------------------------------------

subroutine readinit(dirflexrec,numpoint,grid)

  use mod_var

  implicit none

  character(len=100) :: dirflexrec,filename
  real, parameter :: scaleconc=1.e12
  real, parameter :: smallnum=1.e-38
  integer :: numpoint,itime
  integer :: ierr,ks,kp,sp_count_i,sp_count_r,ii,ir,fact,n,kz,jy,ix
  real, dimension(numxgrid,numygrid,numzgrid,numpoint) :: grid
  integer, dimension(numxgrid*numygrid*numzgrid) :: sparse_dump_i
  real, dimension(numxgrid*numygrid*numzgrid) :: sparse_dump_r

  filename=trim(dirflexrec)//'grid_initial_001'

  write(logid,*) 'Reading initial grid file: '//filename

  grid(:,:,:,:)=0.
  open(100,file=trim(filename),form='unformatted',action='read',status='old',iostat=ierr)
  if(ierr.gt.0) write(logid,*) 'Error opening file: '//filename
  read(100) itime
  do ks=1,nspec
    do kp=1,numpoint
    ! read wet deposition
      read(100) sp_count_i
      read(100) (sparse_dump_i(ix),ix=1,sp_count_i)
      read(100) sp_count_r
      read(100) (sparse_dump_r(ix),ix=1,sp_count_r)
      ! read dry deposition
      read(100) sp_count_i
      read(100) (sparse_dump_i(ix),ix=1,sp_count_i)
      read(100) sp_count_r
      read(100) (sparse_dump_r(ix),ix=1,sp_count_r)
      ! read concentrations
      fact=1
      read(100) sp_count_i
      read(100) (sparse_dump_i(ix),ix=1,sp_count_i)
      read(100) sp_count_r
      read(100) (sparse_dump_r(ix),ix=1,sp_count_r)
      ii=0
      do ir=1,sp_count_r
        if ((sparse_dump_r(ir)*fact).gt.smallnum) then
          ii=ii+1
          n=sparse_dump_i(ii)
          fact=fact*(-1.)
        else
          n=n+1
        endif
        kz=n/(numxgrid*numygrid)
        jy=(n-kz*numxgrid*numygrid)/numxgrid
        ix=n-numxgrid*numygrid*kz-numxgrid*jy
        if(ks.eq.spec) then
          grid(ix+1,jy+1,kz,kp)=abs(sparse_dump_r(ir))*scaleconc
        endif
      end do ! sp_count_r
    end do ! numpoint
  end do ! nspec
  close(100)


end subroutine readinit

