!---------------------------------------------------------------------------------------
! FLEXINVERT : readlsm
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads land-sea mask from a netcdf file
!---------------------------------------------------------------------------------------

subroutine readlsm()

  use mod_var

  implicit none

  include 'netcdf.inc'

  integer :: nid,ierr,xid,yid,zid,lonid,latid,varid
  integer :: iy
  real, dimension(:), allocatable :: lato
  integer, dimension(:), allocatable :: indy
  integer, dimension(:,:,:), allocatable :: tmp

  ! open netcdf file
  write(logid,*) 'Reading file: '//trim(filelsm)
  ierr=nf_open(trim(filelsm),NF_NOWRITE,nid)
  if(ierr.gt.0) then
    write(logid,*) 'Error opening file: '//trim(filelsm)
  endif

  ! inquire about variables
  ierr=nf_inq_dimid(nid,'longitude',xid)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading dimension: longitude'
  endif
  ierr=nf_inq_dimid(nid,'latitude',yid)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading dimension: latitude'
  endif
  ! read dimension sizes
  ierr=nf_inq_dimlen(nid,xid,nxlsm)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading dimension length'
  endif
  ierr=nf_inq_dimlen(nid,yid,nylsm)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading dimension length'
  endif

  allocate(lonlsm(nxlsm))
  allocate(latlsm(nylsm))
  allocate(lsm(nxlsm,nylsm,1))
  allocate(lato(nylsm))
  allocate(indy(nylsm))
  allocate(tmp(nxlsm,nylsm,1))

  ! read variables
  ierr=nf_inq_varid(nid,'longitude',lonid)
  ierr=nf_get_var_real(nid,lonid,lonlsm)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading variable: longitude'
  endif
  ierr=nf_inq_varid(nid,'latitude',latid)
  ierr=nf_get_var_real(nid,latid,latlsm)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading variable: latitude'
  endif
  ierr=nf_inq_varid(nid,trim(lsmname),varid)
  ierr=nf_get_var_int(nid,varid,tmp)
  lsm=real(tmp)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading variable: ',trim(lsmname)
  endif
  ! if latitude order N-S convert to S-N
  lato=latlsm
  call sort(nylsm,lato)
  ! indices to lat
  do iy=1,nylsm
    indy(iy)=minloc(lato,dim=1,mask=lato.eq.latlsm(iy))
  end do
  lsm=lsm(:,indy,:)
  latlsm=lato
  ! convert lsm to value of 0 to 1
  lsm = 1.52597204419215e-05*lsm + 0.499984740279558

  ierr=nf_close(nid)

  deallocate(lato)
  deallocate(indy)
  deallocate(tmp)

end subroutine readlsm
