!---------------------------------------------------------------------------------------
! FLEXINVERT : readobs
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads observations from ascii files
!! each station is in a separate file (generated by the R script: prep_obs.r)
!! selects data for timestep and averages these
!---------------------------------------------------------------------------------------

subroutine readobs(rname,typ,lon,alt,jdi,jdf,jdobs,mobs,sdobs,merr,cnt)

  use mod_var
  use mod_dates

  implicit none
  
  character(len=100) :: filename,dump
  character(len=7) :: rname
  character(len=7) :: stnid
  character(len=2) :: typ
  integer :: i,n,ierr,ierr2,ierr3,flag,yr,mn,dd,hh,mi,hloc,cnt
  integer :: yyyymmdd,hhmiss,alt
  real :: con,lon,sd,err
  real(kind=8) :: julday,jdi,jdf
  real, dimension(1000) :: obs,erobs
  real(kind=8), dimension(1000) :: jdobs
  real :: mobs,sdobs,merr
 
  ! initialise mobs, sdobs
  mobs=0.
  sdobs=0.
  merr=0.
  jdobs=0.

  open(100,file=trim(dirinvert)//'obslist',action='read',status='old',iostat=ierr)
  if(ierr.gt.0) then
    write(logid,*) 'Error listing files in directory' 
    stop
  endif

  ! loop over files in directory
  do while(ierr.eq.0)
    read(100,fmt='(A100)',iostat=ierr,end=5) filename
    if(ierr.gt.0) go to 5
    flag=0
    if(filename(1:7).eq.rname) flag=1

    ! filename is for current receptor/station 
    if(flag.gt.0) then
      open(200,file=trim(dirobs)//trim(filename),action='read',status='old',iostat=ierr2)
      if(ierr2.gt.0) write(logid,*) 'Error opening file: '//trim(dirobs)//trim(filename)
      if(ierr2.eq.0) write(logid,*) 'Reading file: '//trim(dirobs)//trim(filename)

      ! modify the lines below for different observation file formats
      ! header has 21 lines
      do n=1,21
        read(200,fmt='(A100)',iostat=ierr2,end=10) dump
      end do
      i=0
      obs(:)=0.
      erobs(:)=0.
      do while(ierr2.eq.0)
        read(200,fmt='(I4,1X,I2,1X,I2,1X,I2,1X,I2,F10.4,1X,F10.4)',iostat=ierr2,end=10) yr,mn,dd,hh,mi,con,err
        if(con.eq.missval) go to 20
        ! calculate julday 
        yyyymmdd=yr*10000+mn*100+dd
        hhmiss=hh*10000+mi*100
        julday=juldate(yyyymmdd,hhmiss)
        ! if in relevant timestep add to buffer
        if(julday.ge.jdi.and.julday.lt.jdf) then
          if(obsfilter.and.typ.eq.'CM') then
            ! filter observations
            ! local time
            hloc=hh+int(lon*24./360.)
            if(hloc.lt.0) hloc=hloc+24
            if(hloc.gt.24) hloc=hloc-24
            if(alt.gt.1000) then
              ! select nighttime for mountain sites
              if(hloc.lt.6) then
                i=i+1
                obs(i)=con
                if(err.eq.missval) then
                  erobs(i)=measerr
                else
                  erobs(i)=err
                endif
                jdobs(i)=julday
              endif 
            else
              ! select afternoon for low altitude sites
              if(hloc.ge.12.and.hloc.lt.18) then
                i=i+1
                obs(i)=con
                if(err.eq.missval) then
                  erobs(i)=measerr
                else
                  erobs(i)=err
                endif
                jdobs(i)=julday
              endif
            endif
          else
            ! no filtering of observations or flasks
            i=i+1
            obs(i)=con
            if(err.eq.missval) then
              erobs(i)=measerr
            else
              erobs(i)=err
            endif
            jdobs(i)=julday
          endif
        endif ! julday
20      continue
      end do ! ierr
10    continue

      ! average observations and add to observation vector
      if(i.gt.0) then
        mobs=sum(obs(1:i))/float(i)
        merr=sum(erobs(1:i))/float(i)
      endif
      ! set observation error to minimum of measurement error
      if(merr.lt.measerr) merr=measerr
      if(i.gt.1) then
        call stdev(i,obs(1:i),sd)
        sdobs=sd
      endif
      cnt=i
30    continue

      close(200)

    endif ! flag
  end do  ! loop over files
5 continue

  close(100)

end subroutine readobs    
