!---------------------------------------------------------------------------------------
! FLEXINVERT : readsurfinf
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads surface influence from a netcdf file
!---------------------------------------------------------------------------------------

subroutine readsurfinf(surfinf)

  use mod_var 

  implicit none

  include 'netcdf.inc'

  integer :: nid,ierr,xid,yid,varid,lx,ly
  real, dimension(numxgrid,numygrid) :: surfinf

  ! open netcdf file
  write(logid,*) 'Reading file: '//trim(dirauxil)//trim(filesurfinf)
  ierr=nf_open(trim(dirauxil)//trim(filesurfinf),NF_NOWRITE,nid)
  if(ierr.gt.0) then
    write(logid,*) 'Error opening file: '//trim(dirauxil)//trim(filesurfinf)
  endif

  ! inquire about variables
  ierr=nf_inq_dimid(nid,'dimx',xid)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading dimension: dimx'
  endif
  ierr=nf_inq_dimid(nid,'dimy',yid)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading dimension: dimy'
  endif
  ! read dimension sizes
  ierr=nf_inq_dimlen(nid,xid,lx)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading dimension length'
  endif
  ierr=nf_inq_dimlen(nid,yid,ly)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading dimension length'
  endif
  ! check dimensions are correct
  if(lx.ne.numxgrid) then
    write(logid,*) 'Problem x dimension length'
    stop
  endif
  if(ly.ne.numygrid) then
    write(logid,*) 'Problem y dimension length'
    stop
  endif
  ! read variables
  ierr=nf_inq_varid(nid,'surfinf',varid)
  ierr=nf_get_var_real(nid,varid,surfinf)
  if(ierr.gt.0) then
    write(logid,*) 'Error reading variable: surfinf'
  endif

  ierr=nf_close(nid)

end subroutine readsurfinf
