!---------------------------------------------------------------------------------------
! FLEXINVERT : save
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Saves data to netcdf and ascii files
!---------------------------------------------------------------------------------------

subroutine save(outheight,nobs,knest,knest_obs,kout_obs,knest_finobs,&
                &covb,covbfin,covbfinaw,covr,covagg,cova,gain,xb,x,yrec,&
                &ydat,yobs,ymod,ybgi,yerr,ebgb,ebga,modout,dateout,timeout)

  use mod_var
  
  implicit none

  character(len=100) :: fileout,varunit,comment
  character(len=100) :: varname,varname1,varname2,varname3,varname4,varname5
  character(len=20) :: dname1,dname2,dname3
  character(len=20) :: numsfstr
  integer :: n,nobs,ierr
  real, dimension(nzmax) :: outheight
  real, dimension(nobsteps*numrec,nemsteps*nbox+nbkgnd) :: knest
  real, dimension(nobsteps*numrec,nemsteps*nbox) :: knest_obs
  real, dimension(nobsteps*numrec,ncrs) :: kout_obs
  real, dimension(nobsteps*numrec,numxnest*numynest) :: knest_finobs
  real, dimension(nemsteps*nbox+nbkgnd,nemsteps*nbox+nbkgnd) :: cova
  real(kind=8), dimension(nbox*nemsteps+nbkgnd,nbox*nemsteps+nbkgnd) :: covb
  real(kind=8), dimension(numxnest*numynest,numxnest*numynest) :: covbfin,covbfinaw
  real(kind=8), dimension(nbox*nemsteps+nbkgnd,nobs) :: gain
  real(kind=8), dimension(nobs,nobs) :: covagg
  real, dimension(nobs,nobs) :: covr
  real, dimension(nbox*nemsteps+nbkgnd) :: x,xb
  real, dimension(nobsteps*numrec) :: yobs,ymod,ybgi,yerr
  real, dimension(nbkgnd) :: ebgb,ebga
  real, dimension(nemsteps) :: timeout
  real(kind=8), dimension(numday*24/relperiod) :: dateout
  character(len=7), dimension(nobsteps*numrec) :: yrec
  integer, dimension(nobsteps*numrec) :: ydat
  real, dimension(nobs) :: ypos,ybgpos
  real, dimension(numday*24/relperiod,numrec,5) :: modout
 
 
  ! numeric scaling factor as string for file attributes
  write(numsfstr,'(ES12.1)') numscale

  ! coarse grid definition (nbox_xy)

  fileout='nbox_xy.nc'
  varname='nbox_xy'
  varunit='none'
  dname1='dimx'
  dname2='dimy'
  comment='inversion grid definition'
  call write2Dncdf(dirinvert,fileout,varname,varunit,real(nbox_xy),&
                   &numxnest,numynest,dimxnest,dimynest,dname1,dname2,comment)

  ! grid operator (gridoper)

  fileout='grid_operator.nc'
  varname='gridoper'
  varunit='none'
  dname1='dimx'
  dname2='dimy'
  comment='inversion grid definition'
  call write2Dncdf(dirinvert,fileout,varname,varunit,real(gridoper),&
                   &nbox,numxgrid*numygrid,dimbox,dimxy,dname1,dname2,comment)

  ! grid coarse operator (gridcrs)

  fileout='grid_coarse.nc'
  varname='gridcrs'
  varunit='none'
  dname1='dimx'
  dname2='dimy'
  comment='inversion grid definition'
  call write2Dncdf(dirinvert,fileout,varname,varunit,real(gridcrs),&
                   &ncrs,numxgrid*numygrid,dimcrs,dimxy,dname1,dname2,comment)

  ! kernel matrix trimmed to observations (knest)

  fileout='knest_trim.nc'
  varname='kernel'
  varunit='sm3/kg'
  dname1='dimr'
  dname2='dimbx'
  comment='surface influence kernel matrix'
  ! scale by numscale
  knest(:,1:nbox*nemsteps)=knest(:,1:nbox*nemsteps)*numscale
  call write2Dncdf(dirinvert,fileout,varname,varunit,knest(1:nobs,:),&
                  &nobs,nbox*nemsteps+nbkgnd,dimr,dimbx,dname1,dname2,comment)
  ! undo scaling by numscale 
  knest(:,1:nbox*nemsteps)=knest(:,1:nbox*nemsteps)/numscale  

  ! kernel matrix averaged to observation time resolution (knest_obs)

  fileout='knest_obs.nc'
  varname='kernel'
  varunit='sm3/kg'
  dname1='dimi'
  dname2='dimb'
  comment='surface influence kernel matrix'
  call write2Dncdf(dirinvert,fileout,varname,varunit,knest_obs(1:nobsteps*numrec,:)*numscale,&
                  &nobsteps*numrec,nbox*nemsteps,dimi,dimb,dname1,dname2,comment)

  ! kernel matrix averaged to observation time resolution (kout_obs)

  fileout='kout_obs.nc'
  varname='kernel'
  varunit='sm3/kg'
  dname1='dimi'
  dname2='dimb'
  comment='surface influence kernel matrix'
  call write2Dncdf(dirinvert,fileout,varname,varunit,kout_obs(1:nobsteps*numrec,:),&
                  &nobsteps*numrec,ncrs,dimi,dimcrs,dname1,dname2,comment)

  ! kernel matrix at fine spatial resolution and averaged to observation time resolution (knest_finobs)

  fileout='knest_finobs.nc'
  varname='kernel'
  varunit='sm3/kg'
  dname1='dimi'
  dname2='dimb'
  comment='surface influence kernel matrix'
  call write2Dncdf(dirinvert,fileout,varname,varunit,knest_finobs(1:nobsteps*numrec,:)*numscale,&
                  &nobsteps*numrec,numxnest*numynest,dimi,dimxynest,dname1,dname2,comment)

  ! prior error covariance matrix at coarse resolution (covb)

  fileout='covb.nc'
  varname='covb'
  varunit='(kg/m3/s)^2'
  dname1='dim1'
  dname2='dim2'
  comment='prior error covariance matrix variable grid, errors scaled by '//trim(numsfstr)
  call write2Dncdf(dirinvert,fileout,varname,varunit,real(covb,kind=4),&
                  &nbox*nemsteps+nbkgnd,nbox*nemsteps+nbkgnd,dimbx,dimbx,dname1,dname2,comment)

  ! prior error covariance matrix at fine resolution (covbfin)

  fileout='covbfin.nc'
  varname='covb'
  varunit='(kg/m3/s)^2'
  dname1='dim1'
  dname2='dim2'
  comment='prior error covariance matrix fine grid, errors scaled by '//trim(numsfstr)
  call write2Dncdf(dirinvert,fileout,varname,varunit,real(covbfin,kind=4),&
                  &numxnest*numynest,numxnest*numynest,dimxynest,dimxynest,dname1,dname2,comment)

  ! prior error covariance matrix at fine resolution (covbfinaw)

  fileout='covbfinaw.nc'
  varname='covb'
  varunit='(kg/m3/s)^2'
  dname1='dim1'
  dname2='dim2'
  comment='prior error covariance matrix fine grid, area weighted, errors scaled by '//trim(numsfstr)
  call write2Dncdf(dirinvert,fileout,varname,varunit,real(covbfinaw,kind=4),&
                  &numxnest*numynest,numxnest*numynest,dimxynest,dimxynest,dname1,dname2,comment)

  ! observation error covariance matrix (covr)

  fileout='covr.nc'
  varname='covr'
  varunit='vmr^2'
  dname1='dim1'
  dname2='dim2'
  comment='observation error covariance matrix'
  call write2Dncdf(dirinvert,fileout,varname,varunit,covr,&
                  &nobs,nobs,dimr,dimr,dname1,dname2,comment)

  ! aggregation error covariance matrix (covagg)

  fileout='covagg.nc'
  varname='covagg'
  varunit='vmr^2'
  dname1='dim1'
  dname2='dim2'
  comment='aggregation error covariance matrix'
  call write2Dncdf(dirinvert,fileout,varname,varunit,real(covagg,kind=4),&
                  &nobs,nobs,dimr,dimr,dname1,dname2,comment)

  ! prior state vector at coarse resolution (xb)

  open(100,file=trim(dirinvert)//'prior.txt',status='replace',action='write',iostat=ierr)
  do n=1,nemsteps*nbox
    write(100,*) xb(n)*outheight(1)/numscale
  end do
  do n=1,nbkgnd
    write(100,*) xb(nbox*nemsteps+n)
  end do
  close(100)

  ! areas of gridcells in coarse grid (area_box)

  open(100,file=trim(dirinvert)//'area_box.txt',status='replace',action='write',iostat=ierr)
  do n=1,nbox
    write(100,*) area_box(n)
  end do
  close(100)

  ! simulated and background mixing ratios a posteriori

  ypos(:)=0.
  ybgpos(:)=0.
  do n=1,nobs
    ypos(n)=dot_product(knest(n,:),x)
    if(optimizbg) then
      ybgpos(n)=dot_product(knest(n,nbox*nemsteps+1:nbox*nemsteps+nbkgnd),&
                            &x(nbox*nemsteps+1:nbox*nemsteps+nbkgnd))
    endif
  end do

  ! observations and simulations a priori and a posteriori

  open(100,file=trim(dirinvert)//'obsread.txt',status='replace',action='write',iostat=ierr)
  do n=1,nobs
    write(100,fmt='(A7,1X,I10,1X,F9.3,1X,F11.3,1X,F11.3,1X,F9.3,1X,F9.3,1X,F9.3)') &
          &yrec(n),ydat(n),yobs(n),ymod(n),ypos(n),ybgi(n),ybgpos(n),yerr(n)
  end do
  close(100)

  ! posterior state vector on coarse grid (x)

  open(100,file=trim(dirinvert)//'posterior.txt',status='replace',action='write',iostat=ierr)
  do n=1,nemsteps*nbox
    write(100,*) x(n)*outheight(1)/numscale
  end do
  do n=1,nbkgnd
    write(100,*) x(nbox*nemsteps+n)
  end do
  close(100)

  ! prior and posterior background scalars and scalar errors

  open(100,file=trim(dirinvert)//'bgscalars.txt',status='replace',action='write',iostat=ierr)
  do n=1,nbkgnd
     write(100,fmt='(F6.4,1X,F6.4,1X,F6.4,1X,F6.4)') &
           &xb(nbox*nemsteps+n),x(nbox*nemsteps+n),ebgb(n),ebga(n)
  end do
  close(100)

  ! posterior error covariance matrix (cova)

  fileout='cova.nc'
  varname='cova'
  varunit='(kg/m3/s)^2'
  dname1='dim1'
  dname2='dim2'
  comment='posterior error covariance matrix, errors scaled by '//trim(numsfstr)
  call write2Dncdf(dirinvert,fileout,varname,varunit,cova,&
                   &nbox*nemsteps+nbkgnd,nbox*nemsteps+nbkgnd,dimbx,dimbx,&
                   &dname1,dname2,comment)

  ! gain matrix (gain)

  fileout='gain.nc'
  varname='gain'
  varunit='none'
  dname1='dim1'
  dname2='dim2'
  comment='gain matrix of contribution functions'
  call write2Dncdf(dirinvert,fileout,varname,varunit,real(gain,kind=4),&
                   &nbox*nemsteps+nbkgnd,nobs,dimbx,dimr,&
                   &dname1,dname2,comment)

  ! posterior simulated mixing ratios at all timesteps (modout)

  fileout='modout.nc'
  varname1='bgpri'
  varname2='bgpos'
  varname3='ypri'
  varname4='ypos'
  varname5='yerr'
  varunit='vmr'
  dname1='time'
  dname2='receptor'
  comment='atmospheric volume mixing ratios'

  call writemodout(dirinvert,fileout,varname1,varname2,varname3,varname4,varname5,&
                  &varunit,modout,numday*24/relperiod,numrec,dateout,recname(1:numrec),&
                  &dname1,dname2,comment)

  ! prior and posterior fluxes and errors at fine resolution

  fileout='emisflex.nc'
  varname='emis_flex'
  varunit='kg/m2/s'
  dname1='longitude'
  dname2='latitude'
  dname3='time'
  comment='prior emissions regridded to FLEXPART resolution'
  call write3Dncdf(dirinvert,fileout,varname,varunit,emisflex,&
                   &numxgrid,numygrid,ntemi,gridlon,gridlat,timemi,&
                   &dname1,dname2,dname3,comment)

  fileout='analysis.nc'
  varname='emis_post'
  varunit='kg/m2/s'
  dname1='longitude'
  dname2='latitude'
  dname3='time'
  comment='posterior emissions on fine grid'
  call write3Dncdf(dirinvert,fileout,varname,varunit,emispost,&
                   &numxnest,numynest,nemsteps,nestlon,nestlat,timeout,&
                   &dname1,dname2,dname3,comment)

  varname='emis_prior'
  varunit='kg/m2/s'
  comment='prior emissions on fine grid'
  call add3Dncdf(dirinvert,fileout,varname,varunit,emisprior(ixl:ixr,iyl:iyu,:),&
                 &numxnest,numynest,nemsteps,nestlon,nestlat,timeout,&
                 &dname1,dname2,dname3,comment)

  varname='error_post'
  varunit='kg/m2/s'
  comment='posterior errors on fine grid'
  call add3Dncdf(dirinvert,fileout,varname,varunit,errpost,&
                 &numxnest,numynest,nemsteps,nestlon,nestlat,timeout,&
                 &dname1,dname2,dname3,comment)

  varname='error_prior'
  varunit='kg/m2/s'
  comment='prior errors on fine grid'
!  call add2Dncdf(dirinvert,fileout,varname,varunit,errprior(ixl:ixr,iyl:iyu),&
!                 &numxnest,numynest,nestlon,nestlat,&
!                 &dname1,dname2,comment)
  call add3Dncdf(dirinvert,fileout,varname,varunit,errpri,&
                 &numxnest,numynest,nemsteps,nestlon,nestlat,timeout,&
                 &dname1,dname2,dname3,comment)

end subroutine save


