!---------------------------------------------------------------------------------------
! FLEXINVERT : save_forward
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Saves data from forward run to netcdf and ascii files
!---------------------------------------------------------------------------------------

subroutine save_forward(nobs,yrec,ydat,yobs,ymod,ybgi,yerr)

  use mod_var
  
  implicit none

  character(len=100) :: fileout,varunit,comment
  character(len=100) :: varname,varname1,varname2,varname3,varname4
  character(len=20) :: dname1,dname2,dname3
  integer :: n,nobs,ierr
  real, dimension(nzmax) :: outheight
  real, dimension(nobsteps*numrec) :: yobs,ymod,ybgi,yerr
  real, dimension(nemsteps) :: timeout
  character(len=7), dimension(nobsteps*numrec) :: yrec
  integer, dimension(nobsteps*numrec) :: ydat
 
  ! coarse grid definition (nbox_xy)

  fileout='nbox_xy.nc'
  varname='nbox_xy'
  varunit='none'
  dname1='dimx'
  dname2='dimy'
  comment='inversion grid definition'
  call write2Dncdf(dirinvert,fileout,varname,varunit,real(nbox_xy),&
                   &numxnest,numynest,dimxnest,dimynest,dname1,dname2,comment)

  ! observations and simulations a priori

  open(100,file=trim(dirinvert)//'checkprior.txt',status='replace',action='write',iostat=ierr)
  do n=1,nobs
    write(100,fmt='(A7,1X,I10,1X,F9.3,1X,F11.3,1X,F9.3,1X,F9.3)') &
          &yrec(n),ydat(n),yobs(n),ymod(n),ybgi(n),yerr(n)
  end do
  close(100)

  ! prior flux at fine resolution

  fileout='prior.nc'
  varname='emis_flex'
  varunit='kg/m2/s'
  dname1='longitude'
  dname2='latitude'
  dname3='time'
  comment='prior emissions regridded to FLEXPART resolution'
  call write3Dncdf(dirinvert,fileout,varname,varunit,emisflex,&
                   &numxgrid,numygrid,ntemi,gridlon,gridlat,timemi,&
                   &dname1,dname2,dname3,comment)

end subroutine save_forward


