!---------------------------------------------------------------------------------------
! FLEXINVERT : timeseries
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Read gridded flexpart output files to calculate surface influence
!! and multiply with the posterior emissions to simulate concentrations from
!! full spatially and temporally resolved data
!---------------------------------------------------------------------------------------

subroutine timeseries(bkgnd,kout,outheight,time,modout,dateout,allobs,xb,x)

  use mod_var
  use mod_dates

  implicit none

  character(len=100) :: dirflexrec,datestr,filename
  integer :: k,n,nr,nt,ii,i1,i2,ix,jy,kz,it,cnt,ind
  integer :: yyyymmddhh,preyyyymmddhh,yyyymm,preyyyymm,yyyy,mm,dd,hh
  integer :: numpoint,eomday
  real :: scalebg
  real(kind=8) :: jd,jdi,jtime
  real, dimension(numday*24/relperiod,numrec,ncrs) :: bkgnd
  real, dimension(numrec*numday*24/relperiod,ncrs) :: kout
  real, dimension(maxpoint) :: ypri,ypos,yunc,bkgndobs
  real, dimension(nemsteps) :: time
  real, dimension(nzmax) :: outheight
  real, dimension(nbox*nemsteps+nbkgnd) :: xb,x
  real, dimension(numxgrid,numygrid,numzgrid,maxpoint) :: grid,uncert,mgrid,muncert
  real, dimension(numday*24/relperiod,numrec,5) :: modout
  real(kind=8), dimension(numday*24/relperiod) :: dateout
  real(kind=8), dimension(nobsteps,numrec,6) :: allobs
  character(len=10), dimension(maxspec) :: species
  real, dimension(nlatbin) :: clatbin

  if(optimizbg.and.bgmethod.eq.1) then
    do n=1,(nlatbin-1)
      clatbin(n)=0.5*(latbin(n)+latbin(n+1))
    enddo
    clatbin(nlatbin)=0.5*(latbin(nlatbin)+90)
  endif

  do nr=1,numrec
    ind=1
    jd=juldatei
    jdi=juldatei
    ! if optimize background and bgmethod is 1: select optimized scalar
    if(optimizbg.and.bgmethod.eq.1) then
      k=minloc(abs(clatbin(:)-reclat(nr)),dim=1,mask=abs(clatbin(:)-reclat(nr)).eq.minval(abs(clatbin(:)-reclat(nr))))
      scalebg=x(nbox*nemsteps+k)
    endif
    do while (jd.le.juldatef)
      yyyymmddhh=caldate(jd)
      yyyymm=yyyymmddhh/10000
      preyyyymm=yyyymm
      preyyyymmddhh=yyyymmddhh
      write(datestr,'(i6)') yyyymm
      dirflexrec=trim(dirflex)//trim(recname(nr))//'/'//trim(datestr)//'/'
      ! adjust start date for trajectories going back in time
      jd=max(juldatei,(jd-trajlen))
      eomday=calceomday(yyyymm)
      numpoint=eomday*24/relperiod
      if(usedirflex) then
        filename=trim(dirflexrec)//'header_grid_time'
      else
        filename=trim(dirflexrec)//'header'
      endif
      call readheader(filename,numpoint,outheight)
      ! read all grids for trajectories in this directory
      mgrid(:,:,:,:)=0.
      muncert(:,:,:,:)=0.
      cnt=0
      do while(yyyymm.le.preyyyymm)
        call readgrid(dirflexrec,jd,numpoint,grid,uncert)
        mgrid=mgrid+grid
        muncert=muncert+uncert
        jd=jd+float(abs(loutstep))/3600./24.
        if(jd.gt.juldatef) go to 10
        yyyymmddhh=caldate(jd)
        yyyymm=yyyymmddhh/10000
        cnt=cnt+1
      end do
10    continue
      ! convert from ppt mass mixing ratio to volume mixing ratio
      mgrid=mgrid*coef*28.97/molarmass
      muncert=muncert*coef*28.97/molarmass
      ypos(:)=0.
      ypri(:)=0.
      yunc(:)=0.
      do nt=1,numpoint
        ! calculate time index to emissions
        jdi=jdi+float(relperiod)/24.
        yyyymmddhh=caldate(jdi)
        yyyy=yyyymmddhh/1000000
        mm=(yyyymmddhh-yyyy*1000000)/10000
        dd=(yyyymmddhh-yyyy*1000000-mm*10000)/100
        hh=yyyymmddhh-yyyy*1000000-mm*10000-dd*100
        jtime=float(yyyy)+(float(mm)-1.)/12.+(float(dd)-1.)/365.+float(hh)/365./24.
        it=minloc(abs(time(:)-jtime),dim=1,mask=abs(time(:)-jtime).eq.minval(abs(time(:)-jtime)))
        do jy=1,numynest
          do ix=1,numxnest
            if(isnan(emisprior(ixl+ix-1,iyl+jy-1,it)).or.isnan(emispost(ix,jy,it))) go to 20
            ypos(nt)=ypos(nt)+mgrid(ixl+ix-1,iyl+jy-1,1,nt)*emispost(ix,jy,it)/outheight(1)
            ypri(nt)=ypri(nt)+mgrid(ixl+ix-1,iyl+jy-1,1,nt)*emisprior(ixl+ix-1,iyl+jy-1,it)/outheight(1)
            yunc(nt)=yunc(nt)+muncert(ixl+ix-1,iyl+jy-1,1,nt)*emispost(ix,jy,it)/outheight(1)
20          continue
          end do
        end do
        dateout(ind+nt-1)=dble(yyyymmddhh)
        if(bgmethod.eq.1) then
          ! interpolate background from backgroundobs.f90
          ii=minloc(abs(allobs(:,nr,1)-jdi),dim=1,mask=abs(allobs(:,nr,1)-jdi).eq.minval(abs(allobs(:,nr,1)-jdi)))
          i1=max(1,ii-1)
          i2=min(size(allobs(:,1,1)),ii+1)
          bkgndobs(nt)=real(allobs(ii,nr,6)+(allobs(i2,nr,6)-allobs(i1,nr,6))*&
                       &(allobs(ii,nr,1)-jdi)/(allobs(i2,nr,1)-allobs(i1,nr,1)),kind=4)
        endif
      end do
      if(optimizbg) then
        ! using background optimization
        if(bgmethod.eq.2) then
          ! use pre-calculated background from backgroundini.f90
          modout(ind:(ind+numpoint-1),nr,1)=matmul((bkgnd(ind:(ind+numpoint-1),nr,:)+&
                                 &kout((nr-1)*(numday*24/relperiod)+ind:(nr-1)*(numday*24/relperiod)+ind+numpoint-1,:)),&
                                 &xb(nbox*nemsteps+1:nbox*nemsteps+nbkgnd))
          ! include posterior background values
          modout(ind:(ind+numpoint-1),nr,2)=matmul((bkgnd(ind:(ind+numpoint-1),nr,:)+&
                                 &kout((nr-1)*(numday*24/relperiod)+ind:(nr-1)*(numday*24/relperiod)+ind+numpoint-1,:)),&
                                 &x(nbox*nemsteps+1:nbox*nemsteps+nbkgnd))
        else
          ! bgmethod is 1
          modout(ind:(ind+numpoint-1),nr,1)=bkgndobs(1:numpoint)
          ! include posterior background values
          modout(ind:(ind+numpoint-1),nr,2)=bkgndobs(1:numpoint)*scalebg
        endif
      else
        ! no background optimization
        if(bgmethod.eq.2) then
          ! use pre-calculated background from backgroundini.f90
          modout(ind:(ind+numpoint-1),nr,1)=sum(bkgnd(ind:(ind+numpoint-1),nr,:),dim=2)+&
                               &sum(kout((nr-1)*(numday*24/relperiod)+ind:(nr-1)*(numday*24/relperiod)+ind+numpoint-1,:),dim=2)
          modout(ind:(ind+numpoint-1),nr,2)=sum(bkgnd(ind:(ind+numpoint-1),nr,:),dim=2)+&
                               &sum(kout((nr-1)*(numday*24/relperiod)+ind:(nr-1)*(numday*24/relperiod)+ind+numpoint-1,:),dim=2)
        else
          ! bgmethod is 1
          modout(ind:(ind+numpoint-1),nr,1)=bkgndobs(1:numpoint)
          modout(ind:(ind+numpoint-1),nr,2)=bkgndobs(1:numpoint)
        endif
      endif
      modout(ind:(ind+numpoint-1),nr,3)=ypri(1:numpoint)
      modout(ind:(ind+numpoint-1),nr,4)=ypos(1:numpoint)
      modout(ind:(ind+numpoint-1),nr,5)=yunc(1:numpoint)
      ind=ind+numpoint
    end do ! jd
  end do ! numrec


end subroutine timeseries
