!---------------------------------------------------------------------------------------
! FLEXINVERT : write2Dncdf 
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Writes a 2D array to a netcdf file
!---------------------------------------------------------------------------------------

subroutine write2Dncdf(dirout,fileout,varname,varunit,var,nd1,nd2,&
                      &dim1,dim2,dname1,dname2,comment)

  use mod_var

  implicit none

  include 'netcdf.inc'

  character(len=100) :: dirout,fileout,varname,varunit,comment
  character(len=20) :: dname1,dname2
  integer :: nd1,nd2,did1,did2
  real :: dim1(nd1),dim2(nd2)
  integer :: nid,varid,ierr,ind
  integer :: dimids(2)
  real :: var(nd1,nd2)

  ! open netcdf file
  write(logid,*) 'Writing file: '//trim(dirout)//trim(fileout)
  ierr=nf_create(trim(dirout)//trim(fileout),NF_CLOBBER,nid)
  if(ierr.gt.0) then
    write(logid,*) 'Error creating file: ',trim(dirout)//trim(fileout)
  endif

  ! define dimensions
  ierr=nf_def_dim(nid,dname1,nd1,did1)
  ierr=nf_def_dim(nid,dname2,nd2,did2)
  ierr=nf_enddef(nid)

  ! define dimension variables
  ierr=nf_redef(nid)
  ierr=nf_def_var(nid,dname1,NF_FLOAT,1,did1,varid)
  ind=len_trim(dname1)
  ierr=nf_put_att_text(nid,varid,'title',len_trim(dname1),dname1(1:ind))
  ierr=nf_enddef(nid)
  ierr=nf_put_var_real(nid,varid,dim1)
  ierr=nf_redef(nid)
  ind=len_trim(dname2)
  ierr=nf_def_var(nid,dname2,NF_FLOAT,1,did2,varid)
  ierr=nf_put_att_text(nid,varid,'title',len_trim(dname2),dname2(1:ind))
  ierr=nf_enddef(nid)
  ierr=nf_put_var_real(nid,varid,dim2)

  ! define dependent variable
  dimids(:)=(/did1,did2/)
  ierr=nf_redef(nid)
  ierr=nf_def_var(nid,varname,NF_FLOAT,2,dimids,varid)
  ind=len_trim(varname)
  ierr=nf_put_att_text(nid,varid,'title',len_trim(varname),varname(1:ind))
  ind=len_trim(varunit)
  ierr=nf_put_att_text(nid,varid,'unit',len_trim(varunit),varunit(1:ind))
  ierr=nf_enddef(nid)

  ! insert variable
  ierr=nf_put_var_real(nid,varid,var)
  if(ierr.gt.0) then
    write(logid,*) 'Error writing variable '//trim(varname) 
  endif

  ! insert variable attribute
  ierr=nf_redef(nid)
  ind=len_trim(comment)
  ierr=nf_put_att_text(nid,varid,'comment',len_trim(comment),comment(1:ind))
  ierr=nf_enddef(nid)

  ierr=nf_close(nid)

end subroutine write2Dncdf
