!---------------------------------------------------------------------------------------
! FLEXINVERT : writemodout 
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Writes a 2D array to a netcdf file
!---------------------------------------------------------------------------------------

subroutine writemodout(dirout,fileout,varname1,varname2,varname3,varname4,varname5,&
                      &varunit,modout,nd1,nd2,dim1,dim2,&
                      &dname1,dname2,comment)

  use mod_var

  implicit none

  include 'netcdf.inc'

  character(len=100) :: dirout,fileout,varname1,varname2,varname3,varname4,varname5,varunit,comment
  integer :: nd1,nd2,did1,did2,didstr
  integer :: nid,varid1,varid2,varid3,varid4,varid5,ierr,ind
  character(len=20) :: dname1,dname2
  character(len=20) :: dim2(nd2)
  real(kind=8) :: dim1(nd1)
  integer :: dimids(2)
  real :: modout(nd1,nd2,5)

  ! open netcdf file
  write(logid,*) 'Writing file: '//trim(dirout)//trim(fileout)
  ierr=nf_create(trim(dirout)//trim(fileout),NF_CLOBBER,nid)
  if(ierr.gt.0) then
    write(logid,*) 'Error creating file: '//trim(dirout)//trim(fileout)
  endif

  ! define dimensions
  ierr=nf_def_dim(nid,dname1,nd1,did1)
  ierr=nf_def_dim(nid,dname2,nd2,did2)
  ierr=nf_def_dim(nid,'stringLen',20,didstr)
  ierr=nf_enddef(nid)

  ! define dimension variables
  ierr=nf_redef(nid)
  ierr=nf_def_var(nid,dname1,NF_DOUBLE,1,did1,varid1)
  ind=len_trim(dname1)
  ierr=nf_put_att_text(nid,varid1,'title',len_trim(dname1),dname1(1:ind))
  ierr=nf_enddef(nid)
  ierr=nf_put_var_double(nid,varid1,dim1)
  ierr=nf_redef(nid)
  ind=len_trim(dname2)
  ierr=nf_def_var(nid,dname2,NF_CHAR,2,(/didstr,did2/),varid1)
  ierr=nf_put_att_text(nid,varid1,'title',len_trim(dname2),dname2(1:ind))
  ierr=nf_enddef(nid)
  ierr=nf_put_var_text(nid,varid1,dim2)

  ! define 1st dependent variable
  dimids(:)=(/did1,did2/)
  ierr=nf_redef(nid)
  ierr=nf_def_var(nid,varname1,NF_FLOAT,2,dimids,varid1)
  ind=len_trim(varname1)
  ierr=nf_put_att_text(nid,varid1,'title',len_trim(varname1),varname1(1:ind))
  ind=len_trim(varunit)
  ierr=nf_put_att_text(nid,varid1,'unit',len_trim(varunit),varunit(1:ind))
  ierr=nf_enddef(nid)
  ! insert 1st variable
  ierr=nf_put_var_real(nid,varid1,modout(:,:,1))
  ! insert 1st variable attribute
  ierr=nf_redef(nid)
  ind=len_trim(comment)
  ierr=nf_put_att_text(nid,varid1,'comment',len_trim(comment),comment(1:ind))
  ierr=nf_enddef(nid)

  ! define 2nd dependent variable
  ierr=nf_redef(nid)
  ierr=nf_def_var(nid,varname2,NF_FLOAT,2,dimids,varid2)
  ind=len_trim(varname2)
  ierr=nf_put_att_text(nid,varid2,'title',len_trim(varname2),varname2(1:ind))
  ind=len_trim(varunit)
  ierr=nf_put_att_text(nid,varid2,'unit',len_trim(varunit),varunit(1:ind))
  ierr=nf_enddef(nid)
  ! insert variable
  ierr=nf_put_var_real(nid,varid2,modout(:,:,2))
  ! insert variable attribute
  ierr=nf_redef(nid)
  ind=len_trim(comment)
  ierr=nf_put_att_text(nid,varid2,'comment',len_trim(comment),comment(1:ind))
  ierr=nf_enddef(nid)

  ! define 3rd dependent variable
  ierr=nf_redef(nid)
  ierr=nf_def_var(nid,varname3,NF_FLOAT,2,dimids,varid3)
  ind=len_trim(varname3)
  ierr=nf_put_att_text(nid,varid3,'title',len_trim(varname3),varname3(1:ind))
  ind=len_trim(varunit)
  ierr=nf_put_att_text(nid,varid3,'unit',len_trim(varunit),varunit(1:ind))
  ierr=nf_enddef(nid)
  ! insert variable
  ierr=nf_put_var_real(nid,varid3,modout(:,:,3))
  ! insert variable attribute
  ierr=nf_redef(nid)
  ind=len_trim(comment)
  ierr=nf_put_att_text(nid,varid3,'comment',len_trim(comment),comment(1:ind))
  ierr=nf_enddef(nid)
  
  ! define 4th dependent variable
  ierr=nf_redef(nid)
  ierr=nf_def_var(nid,varname4,NF_FLOAT,2,dimids,varid4)
  ind=len_trim(varname4)
  ierr=nf_put_att_text(nid,varid4,'title',len_trim(varname4),varname4(1:ind))
  ind=len_trim(varunit)
  ierr=nf_put_att_text(nid,varid4,'unit',len_trim(varunit),varunit(1:ind))
  ierr=nf_enddef(nid)
  ! insert variable
  ierr=nf_put_var_real(nid,varid4,modout(:,:,4))
  ! insert variable attribute
  ierr=nf_redef(nid)
  ind=len_trim(comment)
  ierr=nf_put_att_text(nid,varid4,'comment',len_trim(comment),comment(1:ind))
  ierr=nf_enddef(nid)

  ! define 5th dependent variable
  ierr=nf_redef(nid)
  ierr=nf_def_var(nid,varname5,NF_FLOAT,2,dimids,varid5)
  ind=len_trim(varname5)
  ierr=nf_put_att_text(nid,varid5,'title',len_trim(varname5),varname5(1:ind))
  ind=len_trim(varunit)
  ierr=nf_put_att_text(nid,varid5,'unit',len_trim(varunit),varunit(1:ind))
  ierr=nf_enddef(nid)
  ! insert variable
  ierr=nf_put_var_real(nid,varid5,modout(:,:,5))
  ! insert variable attribute
  ierr=nf_redef(nid)
  ind=len_trim(comment)
  ierr=nf_put_att_text(nid,varid5,'comment',len_trim(comment),comment(1:ind))
  ierr=nf_enddef(nid)

  ierr=nf_close(nid)

end subroutine writemodout
